#include "license_error.h"
#include "license_logging.h"
#include "xalloc.h"
#include "lua.h" /* For lua status codes */
void _op5lic_error_set(struct op5lic_error_t *err, LicenseStatus status, const char *description, const char *file, unsigned int lineno) {
	if (err == NULL)
		return;

	err->status = status;
	xfree(err->file);
	xfree(err->description);
	err->file = xstrdup(file);
	if (description) {
		xasprintf(&(err->description), "%s: %s", op5lic_strerror(err), description);
	}
	else {
		xasprintf(&(err->description), "%s", op5lic_strerror(err));
	}
	err->lineno = lineno;
}

const char *op5lic_strerror(struct op5lic_error_t *err) {
	if (err == NULL) {
		return "Unknown error";
	}
	switch (err->status) {
		case OP5LIC_STATUS_OK: return "No error";
		case OP5LIC_STATUS_NOT_EXIST: return "License file does not exist";
		case OP5LIC_STATUS_UNSIGNED: return "License is not signed";
		case OP5LIC_STATUS_INVALID_SIGNATURE: return "License signature is invalid";
		case OP5LIC_STATUS_DECODING_ERROR: return "License decoding error";
		case OP5LIC_STATUS_NSS_FAILURE: return "NSS failure";
		case OP5LIC_STATUS_LUA_ERRSYNTAX: return "LUA syntax error when loading license";
		case OP5LIC_STATUS_LUA_ERRMEM: return "LUA memory allocation error";
		case OP5LIC_STATUS_LUA_ERRERR: return "LUA error handling error";
		case OP5LIC_STATUS_LUA_ERRRUN: return "LUA runtime error";
		case OP5LIC_STATUS_INVALID_ENTRYPOINT: return "Invalid entry point";
		case OP5LIC_STATUS_MISSING_METADATA: return "Missing or invalid metadata table";
		case OP5LIC_STATUS_BACKBOX_INVALID: return "Invalid v1/v2 license";
	};
	return "BUG!!! Unknown error";
}

LicenseStatus op5lic_error_from_luastatus(int lua_error) {
	switch (lua_error) {
		case LUA_ERRERR:    return OP5LIC_STATUS_LUA_ERRERR;
		case LUA_ERRMEM:    return OP5LIC_STATUS_LUA_ERRMEM;
		case LUA_ERRSYNTAX: return OP5LIC_STATUS_LUA_ERRSYNTAX;
		case LUA_ERRRUN:    return OP5LIC_STATUS_LUA_ERRRUN;
		case 0:             return OP5LIC_STATUS_OK;
	}
	return OP5LIC_STATUS_OK;
}

void op5lic_error_destroy(struct op5lic_error_t * err) {
	if (err == NULL)
		return;
	xfree(err->file);
	xfree(err->description);
	xfree(err);
}

struct op5lic_error_t *op5lic_error_create(void) {
	struct op5lic_error_t *err;
	err = xcalloc(1, sizeof(*err));
	err->description = NULL;
	err->status = OP5LIC_STATUS_OK;
	return err;
}
